<?php

namespace Drupal\breakpoint_field\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'breakpoint_field_item' field type.
 *
 * @FieldType(
 *   id = "breakpoint_field_item",
 *   label = @Translation("Breakpoints"),
 *   module = "breakpoint_field",
 *   description = @Translation("Makes a breakpoint groups breakpoints available to be selected."),
 *   default_widget = "breakpoint_field_widget",
 *   default_formatter = "breakpoint_field_formatter"
 * )
 */
class BreakpointItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'type' => 'text',
          'size' => 'tiny',
          'not null' => FALSE,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['value'] = DataDefinition::create('string')->setLabel(t('Breakpoint Group'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->get('value')->getValue();
    return $value === NULL || $value === '';
  }

}
