<?php

namespace Drupal\breakpoint_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'breakpoint_field_widget' widget.
 *
 * @FieldWidget(
 *   id = "breakpoint_field_widget",
 *   module = "breakpoint_field",
 *   label = @Translation("Breakpoint widget"),
 *   field_types = {
 *     "breakpoint_field_item"
 *   }
 * )
 */
class BreakpointWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'inline' => TRUE,
      'inline_width' => 125,
      'breakpoint_group' => "",
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = [];
    $options = \Drupal::service('breakpoint.manager')->getGroups();

    $elements['breakpoint_group'] = [
      '#type' => 'select',
      '#title' => $this->t('Breakpoint Group'),
      '#options' => $options,
      '#default_value' => $this->getSetting('breakpoint_group'),
      '#description' => $this->t('Select the breakpoint group.'),
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $state = $this->getSetting('breakpoint_group');
    $summary[] = $this->t('Breakpoint group: @state', ['@state' => $state]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $breakpoints = [];
    $breakpoint_group = $this->getSetting('breakpoint_group');

    if ($breakpoint_group) {
      $breakpoints = array_reverse(\Drupal::service('breakpoint.manager')->getBreakpointsByGroup($breakpoint_group));
    }

    $options = [];

    foreach ($breakpoints as $breakpoint_id => $breakpoint) {
      foreach ($breakpoint->getMultipliers() as $multiplier) {
        $label = $multiplier . ' ' . $breakpoint->getLabel() . ' [' . $breakpoint->getMediaQuery() . ']';
        $options[$breakpoint_id] = $label;
      }
    }

    $element += [
      '#type' => 'select',
      '#title' => $this->t('Breakpoints'),
      '#options' => $options,
      '#element_validate' => [
              [static::class, 'validate'],
      ],
    ];
    return ['value' => $element];
  }

  /**
   * {@inheritdoc}
   */
  public static function validate($element, FormStateInterface $form_state) {
    $value = $element['#value'];
    if (strlen($value) == 0) {
      $form_state->setValueForElement($element, '');
      return;
    }
    if (!preg_match('/^[a-zA-Z0-9_\.]+$/i', $value)) {
      $form_state->setError($element, t("Breakpoint Group must be a text sring no more than 255 characters."));
    }
  }

}
